package edu.upvehu.gbg.babel;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

/**
 * Clase que permite que un objeto contenga los modelos de un conjunto de lenguajes
 * conforme a las especificaciones dadas en la práctica 5 de TAP
 * @author German
 */
public class Babel {
    /**
     * Array para el almacenamiento de los lenguajes 
     */
    private final Language[] LENGUAJES;
    
    /**
     * Constructor del objeto Babel con el array de lenguajes descritos en el fichero dado.
     * Asume que por debajo del directorio indicado está el subdirectorio "txt" conteniendo los
     * textos en cada lenguaje según el convenio de nombrado establecido
     * 
     * @throws IOException 
     */
    Babel(String dir, String descriptor) throws IOException{
        //Trabajamos con un recurso de texto que sabems que está codificado en "latin1"
        try (BufferedReader br=new BufferedReader(new InputStreamReader( new FileInputStream(dir+"/"+descriptor),"latin1"))){
            //Almacenaremos los lenguajes en una lista
            List<Language> lenguajes=new ArrayList<>();
            //Para cada linea del listado construiomos el lenguaje descrito
            for( String linea; (linea=br.readLine())!=null; ) {
                String[] campos=linea.split(" ",2);
                lenguajes.add(new Language(campos[0],campos[1],"udhr/txt/"+campos[0]+".txt"));}
            //Y finalmente los dejamos en el campo de tipo array de lenguajes
            LENGUAJES=lenguajes.toArray(new Language[0]);}}

    /**
     * 
     * @param langId identificador del lenguaje
     * @param n número de lenguajes más proximos al identificado por <code>langId</code> que se quieren obtener 
     * @return el array ordenado con los <code>n</code> lenguajes más próximos al identificado por <code>langId</code>
     */
    public Language[] findMostSimilars(String langId, int n){
        //Rechazamos valores de n inadecuados
        if (n<=0) throw new IllegalArgumentException("No tiene sentido solicitar "+n+" lenguajes.");
        if (n>=LENGUAJES.length) throw new IllegalArgumentException("El número de lengajes solicitado ("+n+") excede de los disponibles.");

        //Tomamos el lenguaje de referencia (rechazando en caso de identificadores incorrectos)
        final Language refLang=getLanguage(langId);
        if (refLang==null) throw new IllegalArgumentException("Identificador de lenguaje ("+langId+") desconocido.");
        
        //Ordenamos el array de lenguajes en función de la similaridad con el de referencia
        Arrays.sort(LENGUAJES, new Comparator<Language>(){
            @Override
            public int compare(Language o1, Language o2) {
                return Double.compare(Language.similarity(refLang, o2), Language.similarity(refLang, o1));
            }
        });
        //Y devolvemos un array copia del completo entre las posiciones 1 y n
        return Arrays.copyOfRange(LENGUAJES,1, n);}
    
    /**
     * Getter para un lenguaje con el identificador <code>langId</code>
     * @param langId el identificador del lenguaje a obtener
     * @return el lenguaje correspondiente al identificador <code>langId</code> o null 
     */
    public Language getLanguage(String langId){
        if (langId!=null) for (Language lang:LENGUAJES) if (langId.equals(lang.getId())) return lang;
        throw new IllegalArgumentException("Identificador de lenguaje ("+langId+") incorrecto.");}
    
}